function Error_Percentage=EC_for_PIEV2(DataSet,numings_per_subject,random_split_ID,approach,rowdiv,coldiv)

%%%%    Written by Liang Chen (lchen AT ieee.org), Jan./2009
%Input:
%   DataSet 		-- the entire face set, in our experiment
%             		   we use 'PIE' only in this experiment
%   numings_per_subject 	-- number of images per subject for trainning
%                               (also as gallary), remainings for testing
%                               for TrainDir='../x/kTrain/', 
%                               (x='PIE')
%                               numbers_per_subject=k
%   random_split_ID	-- The random Train/Test split for the experiment
%			   There are a total of 50 splits provided by UIUC 
% 				http://www.cs.uiuc.edu/homes/dengcai2/Data/data.html
% 			   it may take lots of time if you run all of them. 
%   approach		-- the holistic algorithms used in Electoral College
%                       we can choose here
%                       PCA
%			fisherface
%                       SRDA
%                       S-LDA
%                       S-LPP
%%
%   rowdiv and coldiv       -- we partition the face image into 
%                               a total of rowdiv x coldiv windows
%                               if coldiv is not given, we set coldiv = rowdiv
%                               please make sure 
%                               rowdiv <= 0.5(image height) - 2
%                               rowdiv <= 0.5(image height) - 2 
%			                    (2 is the space in each side I leave for shifting).
%                               (see lines 46 and 47)
%
%
%Output:
%   Error_Percentage      -- The Erorr Rate for this "random" split
%
%
%Special Case: when both rowdiv and coldiv are not given, original approach without perturbing is applied.
%
% Example:
%          Error_Percentage=EC_for_PIEV2('PIE',5,1,'PCA',2)
%	   Error_Percentage=EC_for_PIEV2('PIE',5,1,'PCA')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% Shifts allowed in row and colume directions
shiftr=2;			      %Line 46
shiftc=2;			      %Line 47

if ~exist('rowdiv','var')     %original "approach" without perturbing
    rowdiv=1;
    shiftr=0;
    shiftc=0;
end

if ~exist('coldiv','var')
    coldiv=rowdiv;
end

switch lower(DataSet)
    case {lower('PIE')}       % PIE:
        load('PIE/PIE_64x64.mat');
    otherwise
        error('DataSet does not exist!');
end;


%   TrainDir 		-- the train/test split set

TrainDir=[DataSet,'/',int2str(numings_per_subject),'Train']

%%% Size of images: irow*icol
irow=64;
icol=irow;  %irow x icol is image size

Totalshift=(2*shiftr+1)*(2*shiftc+1);

switch lower(DataSet)
    case {lower('PIE')}       %PIE
        num_subjects=68;
        numimgs_gallary=numings_per_subject*num_subjects;
	totalimages=size(fea,1);
        numimgs_test=totalimages-numimgs_gallary;
    otherwise
        error('DataSet does not exist!');
end;

%%% number of all the images
totalimages=size(fea,1);

fea=reshape(fea',irow,icol,totalimages);  %recover the image : from vector to 2D array


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%I try to find all the correct answers here,
%so that, I can easily check the answers to find out the error rates
AllImagesGnd=sort(gnd);
TotalNumber=size(AllImagesGnd,1);
k=1;
tags(k).start=1;
for i=2:TotalNumber
    if AllImagesGnd(i)>k
	k=k+1;
	tags(k-1).final=i-1;
	tags(k-1).total=tags(k-1).final-tags(k-1).start+1;
	tags(k).start=i;
     end;
end;
k=k+1;
tags(k-1).final=TotalNumber;
tags(k-1).total=tags(k-1).final-tags(k-1).start+1;

%Summary: in PIE, each subject has 170 images, except sub.38 which has only 164 images.

AnswerSheet=false(num_subjects,11554-num_subjects*numings_per_subject);
for i=1:num_subjects
    for j=tags(i).start-(i-1)*numings_per_subject:tags(i).final-i*numings_per_subject
         AnswerSheet(i,j)=true;
    end;
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5

allcorrectansers=0;
allcorrectansersnotie=0;
allwronganserspercentage=0;

warning off;

load([TrainDir,'/',int2str(random_split_ID),'.mat']);
trainIdx=sort(trainIdx);
testIdx=sort(testIdx);
trains=fea(:,:,trainIdx);
tests=fea(:,:,testIdx);

         
totalvotes=zeros(num_subjects,numimgs_test);
        
for r=1:rowdiv
    for c=1:coldiv
                                      
         R0=shiftr+round((irow-2*shiftr)*(r-1)/rowdiv)+1; % We leave shiftr and shiftc in each side for shifting purpose
         C0=shiftc+round((icol-2*shiftc)*(c-1)/coldiv)+1;
         R1=shiftr+round((irow-2*shiftr)*r/rowdiv);
         C1=shiftc+round((icol-2*shiftc)*c/coldiv);
                 
         actualsize=(R1-R0+1)*(C1-C0+1);
         trainsample=(squeeze(reshape(trains(R0:R1,C0:C1,:),actualsize,1,[])))';
	
         trainsample=normr(trainsample);

     %%%%%% in case all the pixels are "0", there will be "NoN" after normr, 
     %%%%%% we shall have to set it 1/sqrt(actualsize)
         TF=isnan(trainsample);
         nanID=find(TF==1);
         if length(nanID)>0
              trainsample(nanID)=1/sqrt(actualsize);
         end;
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



     %%%%%%%%%%%%%%%%%%%%%Choose One of the Following Approaches
                switch lower(approach)
                    case {lower('PCA')}       %1. PCA:
                        [nSmp,nFea]=size(trainsample);
                        ReducedDim=min(nSmp,nFea)-1;
                        [eigvector,eigvalue, elapse] = PCA(trainsample,ReducedDim);
                    case {lower('SRDA')}	  %2. SRDA
                        options = []; 		
                        options.gnd = gnd(trainIdx); 
                        options.ReguType = 'Ridge';
                        options.ReguAlpha = 0.01;
                        [eigvector] = SR_caller(options, trainsample);
                    case {lower('S-LDA')}      %3. S-LDA
                        gnd_Train = gnd(trainIdx);      
                        options = []; 
                        options.Regu = 1;
                        options.ReguAlpha = 0.01;
                        options.ReguType = 'Custom';
                        load(['tensorRs\TensorR',int2str(R1-R0+1),'by',int2str(C1-C0+1),'.mat']);
                        options.regularizerR = regularizerR;
                        [eigvector, eigvalue] = LDA(gnd_Train, options, trainsample);
                    case {lower('S-LPP')}       %4. S-LPP
                        gnd_Train = gnd(trainIdx);      
                        options = []; 
                        options.Metric = 'Cosine'; 
                        options.NeighborMode = 'Supervised'; 
                        options.WeightMode = 'Cosine'; 
                        options.gnd = gnd_Train; 
                        W = constructW(trainsample,options); 
                        options.Regu = 1;
                        options.ReguAlpha = 0.01;
                        options.ReguType = 'Custom';
                        load(['tensorRs\TensorR',int2str(R1-R0+1),'by',int2str(C1-C0+1),'.mat']);
                        options.regularizerR = regularizerR;
                        [eigvector, eigvalue] = LPP(W, options, trainsample);
                    case  {lower('FISHERFACE')}		%5. fisherface
			options = [];
			options.Fisherface = 1;
			gnd_Train = gnd(trainIdx);
			[eigvector, eigvalue] = LDA(gnd_Train, options, trainsample);
                    otherwise
                        error([approach,' does not exist!']);
                end;
     %%%%%%%%%%%%%%%%%%%%%%%%

	%Due to the size of the PIE dataset, there is difficulties in big matrix caculation
	%I changed the order of the projections of test set and gallery set

         VT=zeros(numimgs_test,actualsize);
                
         VT=(squeeze(reshape(tests(R0:R1,C0:C1,:),actualsize,1,[])))';
    	 VT=normr(VT);
            %%%%%% in case all the pixels are "0", there will be "NoN" after normr, 
            %%%%%% we shall have to set it 1/sqrt(actualsize)
                TF=isnan(VT);
                nanID=find(TF==1);
                if length(nanID)>0
                        VT(nanID)=1/sqrt(actualsize);
                end;
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

         TestWindow = VT*eigvector;

         VT=[];

%        VT=zeros(numimgs_gallary*Totalshift,actualsize);
         VT=zeros(numimgs_gallary,actualsize);

         totalshift=0;

	 similaritygallary = realmax*ones(numimgs_gallary,numimgs_test);

         for shiftrow=-shiftr:shiftr
               for  shiftcol=-shiftc:shiftc; 
                        totalshift=totalshift+1;
%                       indL=numimgs_gallary*(totalshift-1)+1;
%                       indH=numimgs_gallary*totalshift;
%                       VT(indL:indH,:)=(squeeze(reshape(trains(R0+shiftrow:R1+shiftrow,C0+shiftcol:C1+shiftcol,:),actualsize,1,[])))';
			VT=(squeeze(reshape(trains(R0+shiftrow:R1+shiftrow,C0+shiftcol:C1+shiftcol,:),actualsize,1,[])))';
			VT=normr(VT);
        	     %%%%%% in case all the pixels are "0", there will be "NoN" after normr, 
                     %%%%%% we shall have to set it 1/sqrt(actualsize)
                	TF=isnan(VT);
                	nanID=find(TF==1);
                	if length(nanID)>0
                    	VT(nanID)=1/sqrt(actualsize);
                    end;
		     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

			GallaryWindow = VT*eigvector;
			VT=[];
                        similaritygallary = min(similaritygallary,EuDist(GallaryWindow,TestWindow));
               end;
         end;
                 

                    
%        tempsim = EuDist(GallaryWindow,TestWindow);
                
%        simt=min(reshape(tempsim,[numimgs_gallary Totalshift numimgs_test]),[],2);
%        tempsim=[];

%        similaritygallary=squeeze(simt);
%        simt=[];
                    
         similaritysubject_tmp= min(reshape(similaritygallary, [numings_per_subject num_subjects  numimgs_test]),[],1);
         similaritygallary=[];
                    
         similaritysubject=squeeze(similaritysubject_tmp);
         similaritysubject_tmp=[];

         M=min(similaritysubject,[],1);
         si=find(similaritysubject<=repmat(M+eps,num_subjects,1));
                                        
         totalvotes(si)=totalvotes(si)+1;    
                    
         similaritysubject=[];
   
         si=[];
         M=[];
   end;
end;
         
MT=max(totalvotes,[],1);
IndexofMT=find(totalvotes==repmat(MT,num_subjects,1));
totalvotes=0*totalvotes;
totalvotes(IndexofMT)=1;
counteachprobe=sum(totalvotes,1);
totalvotes=totalvotes./repmat(counteachprobe,num_subjects,1);
 
totalvotes=totalvotes.*AnswerSheet;
         
nonzeroitems=nonzeros(totalvotes);
rightanswer=sum(nonzeroitems);
rightanswernotie=size(nonzeroitems,1)+sum(sign(nonzeroitems-1));
    
allcorrectanswersnotie=rightanswernotie;
allcorrectanswers=rightanswer;
allwronganswerspercentage=(1-rightanswer/numimgs_test)*100;

Error_Percentage=allwronganswerspercentage;
%Error_Percentage_Std=std(allwronganswerspercentage,1);